from math import log, sqrt

from . import factor

__all__ = [
    'is_composite', 'is_even', 'is_highly_composite', 'is_largely_composite',
    'is_mersenne_prime', 'is_odd', 'is_perfect', 'is_prime', 'is_regular'
]


def is_even(n: int) -> bool:
    return n % 2 == 0


def is_odd(n: int) -> bool:
    return not is_even(n)


def is_prime(n: int) -> bool:
    n = abs(n)
    return n > 1 and all((n % i) for i in range(2, 2 + int(sqrt(n) - 1)))


def is_composite(n: int) -> bool:
    return not is_prime(n)


def is_mersenne_prime(n: int) -> int:
    return log(n + 1, 2).is_integer() and is_prime(n)


def is_perfect(n: int) -> bool:
    return sum(list(factor.factors(n))) == 2 * n


def is_regular(n: int) -> bool:
    return all(p in [2, 3, 5] for p in factor.prime_factors(n))


def is_highly_composite(n: int) -> bool:
    num_factors = len(list(factor.factors(n)))
    for i in range(n):
        if len(list(factor.factors(i))) >= num_factors:
            return False
    return True


def is_largely_composite(n: int) -> bool:
    num_factors = len(list(factor.factors(n)))
    for i in range(n):
        if len(list(factor.factors(i))) > num_factors:
            return False
    return True



from collections import defaultdict
from itertools import islice
from math import log
from typing import Dict, Iterator, List, Optional

__all__ = ['mersenne_primes', 'mersenne_primes_up_to', 'primes', 'primes_up_to']


def primes(n: Optional[int] = None) -> Iterator[int]:
    def infinite_primes() -> Iterator[int]:
        D: Dict[int, List[int]] = defaultdict(list)
        q = 2
        while True:
            if q not in D:
                yield q
                D[q * q] = [q]
            else:
                for p in D[q]:
                    D[p + q].append(p)
                del D[q]
            q += 1
    if n is None:
        return infinite_primes()
    return islice(infinite_primes(), n)


def primes_up_to(n: int) -> Iterator[int]:
    _primes = primes()
    p = next(_primes)
    while p <= n:
        yield p
        p = next(_primes)


def mersenne_primes(n: Optional[int] = None) -> Iterator[int]:
    def infinite_mersenne_primes() -> Iterator[int]:
        D: Dict[int, List[int]] = defaultdict(list)
        q = 2
        while True:
            if q not in D:
                if log(q + 1, 2).is_integer():
                    yield q
                D[q * q] = [q]
            else:
                for p in D[q]:
                    D[p + q].append(p)
                del D[q]
            q += 1
    if n is None:
        return infinite_mersenne_primes()
    return islice(infinite_mersenne_primes(), n)


def mersenne_primes_up_to(n: int) -> Iterator[int]:
    _primes = mersenne_primes()
    p = next(_primes)
    while p <= n:
        yield p
        p = next(_primes)



from pymath.number_theory.primes import *


def test_primes():
    for n1, n2 in zip(primes(10),
                      [2, 3, 5, 7, 11, 13, 17, 19, 23, 29]):
        assert n1 == n2


def test_primes_up_to():
    for n1, n2 in zip(primes_up_to(10),
                      [2, 3, 5, 7]):
        assert n1 == n2



from pymath.counting import *


def test_C():
    assert C(5, 2) == 10
    assert C(7, 4) == 35


def test_P():
    assert P(5, 2) == 20
    assert P(7, 4) == 840


def test_powerset():
    for e1, e2 in zip(powerset([1, 2]),
                      [[], [1], [2], [1, 2]]):
        assert e1 == e2



from pymath.number_theory.number_types import *


def test_is_even():
    assert is_even(2) == True
    assert is_even(0) == True
    assert is_even(1) == False
    assert is_even(-1) == False


def test_is_prime():
    assert is_prime(101) == True
    assert is_prime(0) == False
    assert is_prime(1) == False
    assert is_prime(2) == True
    assert is_prime(7) == True
    assert is_prime(-7) == True


def test_is_mersenne_prime():
    assert is_mersenne_prime(3) == True
    assert is_mersenne_prime(4) == False
    assert is_mersenne_prime(11) == False


def test_is_perfect():
    assert is_perfect(6) == True
    assert is_perfect(5) == False
    assert is_perfect(1) == False


def test_is_regular():
    assert is_regular(48) == True
    assert is_regular(75) == True
    assert is_regular(14) == False


def test_is_highly_composite():
    assert is_highly_composite(60) == True
    assert is_highly_composite(59) == False
    assert is_highly_composite(3) == False
    assert is_highly_composite(2) == True
    assert is_highly_composite(1) == True


def test_is_largely_composite():
    assert is_largely_composite(60) == True
    assert is_largely_composite(5) == False
    assert is_largely_composite(3) == True
    assert is_largely_composite(2) == True
    assert is_largely_composite(1) == True



from pymath.number_theory.analytic import *


def test_goldbach():
    assert goldbach(1) == None
    assert goldbach(11) == None
    assert goldbach(4) == (2, 2)
    assert goldbach(72) == (5, 67)
    assert goldbach(16) == (3, 13)


def test_goldbach_triple():
    assert goldbach_triple(5) == None
    assert goldbach_triple(10) == (2, 3, 5)
    assert goldbach_triple(500) == (2, 7, 491)
    assert goldbach_triple(71) == (2, 2, 67)



#!/usr/bin/env python

from setuptools import find_packages, setup

setup(
    name='pymath2',
    version='1.0.0',
    packages=find_packages(exclude=('tests',)),
    scripts=['scripts/pymath'],
    description='Easy calculations on the command line with Python',
    url='https://github.com/cjbassi/pymath',
)



from math import factorial
from typing import Collection, Iterator, List, TypeVar

__all__ = ['Choose', 'Permute', 'powerset']

T = TypeVar('T')


def Choose(n: int, k: int) -> int:
    return factorial(n) // (factorial(n - k) * factorial(k))


def Permute(n: int, k: int) -> int:
    return factorial(n) // (factorial(n - k))


def powerset(collection: Collection[T]) -> Iterator[List[T]]:
    def powerset_impl(seq):
        if len(seq) == 0:
            yield []
        else:
            for item in powerset(seq[1:]):
                yield item
                yield [seq[0]] + item
    return powerset_impl(list(collection))



from pymath.number_theory.factor import *


def test_factors():
    for n1, n2 in zip(factors(50),
                      [1, 2, 5, 10, 25, 50]):
        assert n1 == n2
    for _ in factors(0):
        assert False
    for n1, n2 in zip(factors(-50),
                      [1, 2, 5, 10, 25, 50]):
        assert n1 == n2


def test_lcm():
    assert lcm(60, 18) == 180
    assert lcm(60, 18, 3) == 180


def test_gcd():
    assert gcd(60, 18) == 6
    assert gcd(60, 18, 3) == 3


def test_extended_lcm():
    assert extended_lcm(5, 2) == [10, 2, 5]
    assert extended_lcm(5, 2, 3) == [30, 6, 15, 10]


def test_extended_gcd():
    assert extended_gcd(10, 6) == (2, -1, 2)


def test_prime_factors():
    for n1, n2 in zip(prime_factors(5), [5]):
        assert n1 == n2
    for n1, n2 in zip(prime_factors(60), [2, 2, 3, 5]):
        assert n1 == n2


def test_largest_prime_factor():
    assert largest_prime_factor(60) == 5
    assert largest_prime_factor(11) == 11


def test_coprime():
    assert are_coprime(5, 2) == True
    assert are_coprime(2, 2) == False
    assert are_coprime(2, 1) == True
    assert are_coprime(2, 0) == False
    assert are_coprime(0, 0) == False
    assert are_coprime(10, 2, 4) == False
    assert are_coprime(10, 3, 4) == True


def test_phi():
    assert phi(1) == 1
    assert phi(2) == 1
    assert phi(3) == 2
    assert phi(10) == 4
    assert phi(16) == 8


def test_coprimes():
    for n1, n2 in zip(coprimes(1), [1]):
        assert n1 == n2
    for n1, n2 in zip(coprimes(10), [1, 3, 7, 9]):
        assert n1 == n2
    for n1, n2 in zip(coprimes(16), [1, 3, 5, 7, 9, 11, 13, 15]):
        assert n1 == n2
